@@ -8,12 +8,27 @@ import hashlib
 import hmac
 import json
 import re
-from typing import Any, Dict, List, Mapping, MutableMapping, Optional, Sequence, Tuple, Union
+from typing import (
+    Any,
+    Dict,
+    Iterator,
+    List,
+    Mapping,
+    MutableMapping,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+)
 from urllib import parse
 
 from meilisearch._httprequests import HttpRequests
 from meilisearch.config import Config
-from meilisearch.errors import MeilisearchError
+from meilisearch.errors import (  # noqa: F401
+    MeilisearchApiError,
+    MeilisearchCommunicationError,
+    MeilisearchError,
+)
 from meilisearch.index import Index
 from meilisearch.models.key import Key, KeysResults
 from meilisearch.models.task import Batch, BatchResults, Task, TaskInfo, TaskResults
@@ -28,12 +43,16 @@ class Client:
     Meilisearch and its permissions.
     """
 
+    # Import aliases to satisfy pylint (used in docstrings)
+    MeilisearchApiError = MeilisearchApiError
+
     def __init__(
         self,
         url: str,
         api_key: Optional[str] = None,
         timeout: Optional[int] = None,
         client_agents: Optional[Tuple[str, ...]] = None,
+        custom_headers: Optional[Mapping[str, str]] = None,
     ) -> None:
         """
         Parameters
@@ -48,10 +67,13 @@ class Client:
         client_agents (optional):
             Used to send additional client agent information for clients extending the functionality
             of this client.
+        custom_headers (optional):
+            Custom headers to add when sending data to Meilisearch.
         """
+
         self.config = Config(url, api_key, timeout=timeout, client_agents=client_agents)
 
-        self.http = HttpRequests(self.config)
+        self.http = HttpRequests(self.config, custom_headers)
 
         self.task_handler = TaskHandler(self.config)
 
@@ -229,6 +251,8 @@ class Client:
         queries:
             List of dictionaries containing the specified indexes and their search queries
             https://www.meilisearch.com/docs/reference/api/search#search-in-an-index
+            It can also include remote options in federationOptions for each query
+            https://www.meilisearch.com/docs/reference/api/network
         federation: (optional):
             Dictionary containing offset and limit
             https://www.meilisearch.com/docs/reference/api/multi_search
@@ -248,6 +272,33 @@ class Client:
             body={"queries": queries, "federation": federation},
         )
 
+    def update_documents_by_function(
+        self, index_uid: str, queries: Dict[str, List[Dict[str, Any]]]
+    ) -> Dict[str, Any]:
+        """Update Documents by function
+        Parameters
+        ----------
+        index_uid:
+            The index_uid where you want to update documents of.
+        queries:
+            List of dictionaries containing functions with or without filters that you want to use to update documents.
+
+        Returns
+        -------
+        task_info:
+            TaskInfo instance containing information about a task to track the progress of an asynchronous process.
+            https://www.meilisearch.com/docs/reference/api/tasks#get-one-task
+
+        Raises
+        ------
+        MeilisearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
+        """
+        return self.http.post(
+            path=f"{self.config.paths.index}/{index_uid}/{self.config.paths.document}/{self.config.paths.edit}",
+            body=dict(queries),
+        )
+
     def get_all_stats(self) -> Dict[str, Any]:
         """Get all stats of Meilisearch
 
@@ -725,6 +776,214 @@ class Client:
 
         return jwt_token
 
+    def add_or_update_networks(self, body: Union[MutableMapping[str, Any], None]) -> Dict[str, str]:
+        """Set all the Remote Networks
+
+        Parameters
+        ----------
+        body:
+            Remote networks that are allowed
+
+        Returns
+        -------
+        remote networks:
+            Remote Networks containing information about the networks allowed/present.
+            https://www.meilisearch.com/docs/reference/api/network
+
+        Raises
+        ------
+        MeilisearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
+        """
+
+        return self.http.patch(path=f"{self.config.paths.network}", body=body)
+
+    def get_all_networks(self) -> Dict[str, str]:
+        """Fetches all the remote-networks present
+
+        Returns
+        -------
+        remote networks:
+            All remote networks containing information about each remote and their respective remote-name and searchApi key
+
+        Raises
+        ------
+        MeilisearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
+        """
+        return self.http.get(path=f"{self.config.paths.network}")
+
+    def create_chat_completion(
+        self,
+        workspace_uid: str,
+        messages: List[Dict[str, str]],
+        model: str = "gpt-3.5-turbo",
+        stream: bool = True,
+    ) -> Iterator[Dict[str, Any]]:
+        """Streams a chat completion from the Meilisearch chat API.
+
+        Parameters
+        ----------
+        workspace_uid:
+            Unique identifier of the chat workspace to use.
+        messages:
+            List of message dicts (e.g. {"role": "user", "content": "..."}) comprising the chat history.
+        model:
+            The model name to use for completion (should correspond to the LLM in workspace settings).
+        stream:
+            Whether to stream the response. Must be True for now (only streaming is supported).
+
+        Returns
+        -------
+        chunks:
+            Parsed chunks of the completion as Python dicts. Each chunk is a partial response (in OpenAI format).
+            Iteration ends when the completion is done.
+
+        Raises
+        ------
+        MeilisearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
+        MeilisearchCommunicationError
+            If a network error occurs.
+        ValueError
+            If stream=False is passed (not currently supported), or if workspace_uid is empty or contains path separators.
+        """
+        if not stream:
+            # The API currently only supports streaming responses:
+            raise ValueError("Non-streaming chat completions are not supported. Use stream=True.")
+
+        # Basic security validation (only what's needed)
+        if not workspace_uid:
+            raise ValueError("workspace_uid is required and cannot be empty")
+        if "/" in workspace_uid or "\\" in workspace_uid:
+            raise ValueError("Invalid workspace_uid: must not contain path separators")
+
+        payload = {"model": model, "messages": messages, "stream": True}
+
+        # Construct the URL for the chat completions route.
+        endpoint = f"chats/{workspace_uid}/chat/completions"
+
+        # Initiate the HTTP POST request in streaming mode.
+        response = self.http.post_stream(endpoint, body=payload)
+
+        try:
+            # Iterate over the streaming response lines
+            for raw_line in response.iter_lines():
+                if raw_line is None or raw_line == b"":
+                    continue
+
+                line = raw_line.decode("utf-8")
+                if line.startswith("data: "):
+                    data = line[len("data: ") :]
+                    if data.strip() == "[DONE]":
+                        break
+
+                    try:
+                        chunk = json.loads(data)
+                        yield chunk
+                    except json.JSONDecodeError as e:
+                        raise MeilisearchCommunicationError(
+                            f"Failed to parse chat chunk: {e}"
+                        ) from e
+        finally:
+            response.close()
+
+    def get_chat_workspaces(
+        self,
+        *,
+        offset: Optional[int] = None,
+        limit: Optional[int] = None,
+    ) -> Dict[str, Any]:
+        """Get all chat workspaces.
+
+        Parameters
+        ----------
+        offset (optional):
+            Number of workspaces to skip.
+        limit (optional):
+            Maximum number of workspaces to return.
+
+        Returns
+        -------
+        workspaces
+            Dictionary containing the list of chat workspaces and pagination information.
+
+        Raises
+        ------
+        MeilisearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
+        """
+        params = {}
+        if offset is not None:
+            params["offset"] = offset
+        if limit is not None:
+            params["limit"] = limit
+        path = "chats" + ("?" + parse.urlencode(params) if params else "")
+        return self.http.get(path)
+
+    def get_chat_workspace_settings(self, workspace_uid: str) -> Dict[str, Any]:
+        """Get the settings for a specific chat workspace.
+
+        Parameters
+        ----------
+        workspace_uid:
+            Unique identifier of the chat workspace.
+
+        Returns
+        -------
+        settings:
+            Dictionary containing the workspace settings.
+
+        Raises
+        ------
+        MeilisearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
+        ValueError
+            If workspace_uid is empty or contains path separators.
+        """
+        # Basic security validation (only what's needed)
+        if not workspace_uid:
+            raise ValueError("workspace_uid is required and cannot be empty")
+        if "/" in workspace_uid or "\\" in workspace_uid:
+            raise ValueError("Invalid workspace_uid: must not contain path separators")
+
+        return self.http.get(f"chats/{workspace_uid}/settings")
+
+    def update_chat_workspace_settings(
+        self, workspace_uid: str, settings: Mapping[str, Any]
+    ) -> Dict[str, Any]:
+        """Update the settings for a specific chat workspace.
+
+        Parameters
+        ----------
+        workspace_uid:
+            Unique identifier of the chat workspace.
+        settings:
+            Dictionary containing the settings to update.
+
+        Returns
+        -------
+        settings:
+            Dictionary containing the updated workspace settings.
+
+        Raises
+        ------
+        MeilisearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
+        ValueError
+            If workspace_uid is empty or contains path separators, or if settings is empty.
+        """
+        # Basic security validation (only what's needed)
+        if not workspace_uid:
+            raise ValueError("workspace_uid is required and cannot be empty")
+        if "/" in workspace_uid or "\\" in workspace_uid:
+            raise ValueError("Invalid workspace_uid: must not contain path separators")
+
+        if not settings:
+            raise ValueError("settings cannot be empty")
+
+        return self.http.patch(f"chats/{workspace_uid}/settings", body=settings)
+
     @staticmethod
     def _base64url_encode(data: bytes) -> str:
         return base64.urlsafe_b64encode(data).decode("utf-8").replace("=", "")
